@@ -84,19 +84,20 @@ class TickerDax:
             self._start_redis_server()
 
     @staticmethod
-    def _get_cache_keys(route, symbols, timestamps):
+    def _get_cache_keys(route, symbols, timestamps, timeframe):
         """
         Get all the cache keys.
 
         :param str route: The data route.
         :param list[str] symbols: A list of symbols.
         :param list[float] timestamps: The timestamps to get.
+        :param str timeframe: The timeframe interval i.e. 1m,15m,30m,1h,4h,1d, etc.
         :returns: A complete list of all the cache keys.
         :rtype list
         """
         keys = []
         for symbol in symbols:
-            keys.extend([f'{NAME}/{route}/{symbol}/{timestamp}' for timestamp in timestamps])
+            keys.extend([f'{NAME}/{route}/{symbol}/{timeframe}/{timestamp}' for timestamp in timestamps])
         return keys
 
     @staticmethod
@@ -105,7 +106,7 @@ class TickerDax:
         Normalizes the route format.
 
         :param str route: The data route.
-        :returns: A normalizedd the route.
+        :returns: A normalized route.
         :rtype str
         """
         return route.strip('/').strip('\\').replace('\\', '/')
@@ -137,28 +138,31 @@ class TickerDax:
         rest_requests = {}
         for key, cache_value in zip(keys, cache_values):
             items = key.split('/')
-            symbol = items[-2]
+            symbol = items[-3]
             timestamp = float(items[-1])
 
+            # if the cache value is not a dict load it from json
+            cache_value = json.loads(cache_value) if cache_value else {}
+
             if not cache_value:
                 if not rest_requests.get(symbol):
                     rest_requests[symbol] = []
                 rest_requests[symbol].append(timestamp)
 
-            # if the data is nothing more than the symbol and id, then this should be
-            # marked as missing data.
-            elif json.loads(cache_value) == {'id': timestamp, 'symbol': symbol}:
+            # if there is no data, then this should be marked as missing data.
+            elif not cache_value.get('data'):
                 # if force is true then request the already reported missing values
                 if self._force:
                     if not rest_requests.get(symbol):
                         rest_requests[symbol] = []
                     rest_requests[symbol].append(timestamp)
                 else:
-                    self.missing_values.append(json.loads(cache_value))
+
+                    self.missing_values.append(cache_value)
 
             # otherwise, the data is already cached
             else:
-                self.cached_values.append(json.loads(cache_value))
+                self.cached_values.append(cache_value)
 
         # return the needed rest requests
         return rest_requests
@@ -286,13 +290,14 @@ class TickerDax:
                 return
         raise self.report_error('TickerDax failed to connect to the redis server')
 
-    def _batch_request(self, route, symbol, timestamps):
+    def _batch_request(self, route, symbol, timestamps, timeframe):
         """
         Batches requests until all timestamps are retrieved.
 
         :param str route: The data route.
         :param str symbol: The symbol to get.
         :param list[float] timestamps: The timestamps to get.
+        :param str timeframe: The timeframe interval i.e. 1m,15m,30m,1h,4h,1d, etc.
         :returns A list of all the responses to the request.
         :rtype list[dict]
         """
@@ -303,8 +308,10 @@ class TickerDax:
             batch.append(key)
 
             if index % self._batch_size == 0:
-                self._logger.info(f'batch requesting {index}/{number_of_timestamp} "{route}/{symbol}" timestamps...')
-                result.extend(self._rest_request(route, symbol, batch))
+                self._logger.info(
+                    f'batch requesting {index}/{number_of_timestamp} "{route}/{symbol}" {timeframe} timestamps...'
+                )
+                result.extend(self._rest_request(route, symbol, batch, timeframe))
 
                 # clear the batch
                 batch.clear()
@@ -312,20 +319,22 @@ class TickerDax:
         # get any remaining items in the last batch
         if batch:
             self._logger.info(
-                f'batch requesting {number_of_timestamp}/{number_of_timestamp} "{route}/{symbol}" timestamps...'
+                f'batch requesting {number_of_timestamp}/{number_of_timestamp} "{route}/{symbol}" '
+                f'{timeframe} timestamps...'
             )
-            result.extend(self._rest_request(route, symbol, batch))
+            result.extend(self._rest_request(route, symbol, batch, timeframe))
 
         self._logger.debug(f'batch "{route}/{symbol}" requests complete!')
         return result
 
-    async def _async_batch_request(self, route, symbol, timestamps):
+    async def _async_batch_request(self, route, symbol, timestamps, timeframe):
         """
         Batches requests until all timestamps are retrieved.
 
         :param str route: The data route.
         :param str symbol: The symbol to get.
         :param list[float] timestamps: The timestamps to get.
+        :param str timeframe: The timeframe interval i.e. 1m,15m,30m,1h,4h,1d, etc.
         :returns A list of all the responses to the request.
         :rtype list[dict]
         """
@@ -337,7 +346,7 @@ class TickerDax:
 
             if index % self._batch_size == 0:
                 self._logger.info(f'batch requesting {index}/{number_of_timestamp} "{route}/{symbol}" timestamps...')
-                result.extend(self._rest_request(route, symbol, batch))
+                result.extend(self._rest_request(route, symbol, batch, timeframe))
 
                 # clear the batch
                 batch.clear()
@@ -347,7 +356,7 @@ class TickerDax:
             self._logger.info(
                 f'batch requesting {number_of_timestamp}/{number_of_timestamp} "{route}/{symbol}" timestamps...'
             )
-            result.extend(self._rest_request(route, symbol, batch))
+            result.extend(self._rest_request(route, symbol, batch, timeframe))
 
         self._logger.debug(f'batch "{route}/{symbol}" requests complete!')
         return result
@@ -360,7 +369,7 @@ class TickerDax:
         :param str route: The data route.
         :param list[str] symbols: A list of symbols.
         """
-        uri = f'{self._host}?route={route}&symbols={symbols}'
+        uri = f'{self._host}?route={route}&symbols={",".join(symbols)}'
         try:
             async with websockets.connect(
                     uri,
@@ -370,9 +379,10 @@ class TickerDax:
                 while True:
                     data = json.loads(await connected_socket.recv())
                     symbol = data.get('symbol')
-                    timestamp = data.get('id')
+                    timestamp = data.get('timestamp')
+                    timeframe = data.get('timeframe')
                     if symbol and timestamp:
-                        for key in self._get_cache_keys(self._format_route(route), [symbol], [timestamp]):
+                        for key in self._get_cache_keys(self._format_route(route), [symbol], [timestamp], timeframe):
                             self.redis_client.set(key, json.dumps(data))
                             self._logger.info(f'Cached: {pformat(data)}')
 
@@ -390,23 +400,28 @@ class TickerDax:
                 )
 
     # async def _rest_request(self, route, symbol, timestamps):
-    def _rest_request(self, route, symbol, timestamps):
+    def _rest_request(self, route, symbol, timestamps, timeframe):
         """
         Preforms a single REST request.
 
         :param str route: The data route.
         :param str symbol: The symbol to get.
         :param list[float] timestamps: The timestamps to get.
+        :param str timeframe: The timeframe interval i.e. 1m,15m,30m,1h,4h,1d, etc.
         :return list[dict]: A list of dictionaries as the response to the request.
         """
+        if len(timestamps) == 1:
+            timestamps = [timestamps[0], timestamps[0]]
+
         try:
             response = requests.get(
-                f'{self._rest_api_host}/{self._rest_api_version}/{route}',
+                f'{self._rest_api_host}/{self._rest_api_version}/{route}/{symbol}',
                 headers={"x-api-key": self._rest_api_key},
-                data=json.dumps({
-                    'symbol': symbol,
-                    'timestamps': timestamps
-                })
+                params={
+                    'since': timestamps[0],
+                    'till': timestamps[-1],
+                    'timeframe': timeframe
+                }
             )
             if response.ok:
                 return response.json()
@@ -425,13 +440,14 @@ class TickerDax:
         except Exception as error:
             self.report_error(str(error))
 
-    def _request(self, route, rest_requests):
+    def _request(self, route, rest_requests, timeframe):
         """
         A request to first the local cache, then to the REST API if data is missing in the
         cache.
 
         :param str route: The data route.
         :param dict rest_requests: A dictionary of symbols and timestamps.
+        :param str timeframe: The timeframe interval i.e. 1m,15m,30m,1h,4h,1d, etc.
         :return list[dict]: A list of dictionaries as the response to the request.
         """
         rest_values = []
@@ -439,16 +455,17 @@ class TickerDax:
             self._logger.info(f'Requesting {route} data from REST API...')
             # gather the symbols concurrently
             for symbol, timestamps in rest_requests.items():
-                rest_values.extend(self._batch_request(route, symbol, timestamps))
+                rest_values.extend(self._batch_request(route, symbol, timestamps, timeframe))
         return rest_values
 
-    async def _async_request(self, route, rest_requests):
+    async def _async_request(self, route, rest_requests, timeframe):
         """
         A request to first the local cache, then to the REST API if data is missing in the
         cache.
 
         :param str route: The data route.
         :param dict rest_requests: A dictionary of symbols and timestamps.
+        :param str timeframe: The timeframe interval i.e. 1m,15m,30m,1h,4h,1d, etc.
         :return list[dict]: A list of dictionaries as the response to the request.
         """
         rest_values = []
@@ -456,7 +473,8 @@ class TickerDax:
             self._logger.info(f'Requesting {route} data from REST API...')
             # gather the symbols concurrently
             for result in await asyncio.gather(*[
-                self._batch_request(route, symbol, timestamps) for symbol, timestamps in rest_requests.items()
+                self._batch_request(route, symbol, timestamps, timeframe)
+                for symbol, timestamps in rest_requests.items()
             ]):
                 rest_values.extend(result)
         return rest_values
@@ -468,16 +486,15 @@ class TickerDax:
         :param dict routes: A dictionary of route names and their symbols.
         """
         await asyncio.gather(*[self._stream_to_cache(
-                f'/{self._format_route(route)}', ','.join(symbols)) for route, symbols in routes.items()
+                f'/{self._format_route(route)}', symbols) for route, symbols in routes.items()
         ])
 
-    def _update_cache(self, route, keys, timeframe):
+    def _update_cache(self, route, keys):
         """
         Saves any new data from the response to the cache.
 
         :param str route: The data route.
         :param list[str] keys: A complete list of all the cache keys.
-        :param str timeframe: The time interval.
         :returns: The combined result of cache values, rest values, and blank values.
         :rtype list
         """
@@ -485,8 +502,9 @@ class TickerDax:
         # remove all the cache keys that already had a cached value
         for cached_value in self.cached_values:
             symbol = cached_value.get('symbol')
-            timestamp = cached_value.get('id')
-            key = f'{NAME}/{route}/{symbol}/{timestamp}'
+            timestamp = cached_value.get('timestamp')
+            timeframe = cached_value.get('timeframe')
+            key = f'{NAME}/{route}/{symbol}/{timeframe}/{timestamp}'
             if key in keys:
                 keys.remove(key)
 
@@ -495,8 +513,9 @@ class TickerDax:
         # cache the rest values
         for rest_value in self.rest_values:
             symbol = rest_value.get('symbol')
-            timestamp = rest_value.get('id')
-            key = f'{NAME}/{route}/{symbol}/{timestamp}'
+            timestamp = rest_value.get('timestamp')
+            timeframe = rest_value.get('timeframe')
+            key = f'{NAME}/{route}/{symbol}/{timeframe}/{timestamp}'
 
             self.redis_client.set(key, json.dumps(rest_value))
             # remove the key now that it is cached
@@ -507,10 +526,11 @@ class TickerDax:
         # if there are any remaining keys, then that means they were missing from the rest api
         for key in keys:
             items = key.split('/')
-            symbol = items[-2]
+            symbol = items[-3]
+            timeframe = items[-2]
             timestamp = float(items[-1])
 
-            missing_value = {'id': timestamp, 'symbol': symbol}
+            missing_value = {'timestamp': timestamp, 'symbol': symbol, 'timeframe': timeframe}
             result.append(missing_value)
 
             # this will set the missing value in the cache with an expiration time that matches the given timeframe
@@ -547,22 +567,17 @@ class TickerDax:
         :returns: A list of all available routes from the REST api.
         :rtype: list
         """
-        ignored_routes = ['info/plans']
-        routes = []
-        for route in requests.get(f'{self._rest_api_host}/openapi.json').json().get('paths', {}).keys():
-            route = route.replace(f'/{self._rest_api_version}/', '')
-            if route not in ignored_routes:
-                routes.append(route)
-        return routes
+        response = requests.get(f'{self._rest_api_host}/{self._rest_api_version}/info/routes')
+        return response.json()
 
-    def get_route(self, route, symbols, start, end, timeframe='1h', asynchronous=False):
+    def get_route(self, route, symbols, since, till, timeframe='1h', asynchronous=False):
         """
         Get data for a route and it's symbols between the start and end times and at the timeframe interval.
 
         :param str route: The data route.
         :param list[str] symbols: A list of symbols.
-        :param datetime start: The UTC start time.
-        :param datetime end: The UTC end time.
+        :param datetime since: The UTC start time.
+        :param datetime till: The UTC end time.
         :param str timeframe: The time interval.
         :param bool asynchronous: Whether the request is asynchronous.
         :returns: The sorted result.
@@ -573,8 +588,8 @@ class TickerDax:
         self.missing_values.clear()
 
         route = self._format_route(route)
-        timestamps = formatting.get_timestamp_range(start, end, timeframe)
-        keys = self._get_cache_keys(route, symbols, timestamps)
+        timestamps = formatting.get_timestamp_range(since, till, timeframe)
+        keys = self._get_cache_keys(route, symbols, timestamps, timeframe)
 
         # get the cached values and determine which rest requests are outstanding
         self._logger.debug(f'Checking "{route}" cache for {symbols}...')
@@ -582,12 +597,12 @@ class TickerDax:
 
         # make the request asynchronously or synchronously
         if asynchronous:
-            self.rest_values = asyncio.run(self._async_request(route, outstanding_rest_requests))
+            self.rest_values = asyncio.run(self._async_request(route, outstanding_rest_requests, timeframe))
         else:
-            self.rest_values = self._request(route, outstanding_rest_requests)
+            self.rest_values = self._request(route, outstanding_rest_requests, timeframe)
 
-        result = self._update_cache(route, keys, timeframe)
-        return sorted(result, key=lambda i: i['id'])
+        result = self._update_cache(route, keys)
+        return sorted(result, key=lambda i: i['timestamp'])
 
     def report_error(self, message):
         """
@@ -616,14 +631,15 @@ if __name__ == '__main__':
     client = TickerDax()
 
     pprint(client.get_route(
-        route='predictions/ml-order-book-model-1',
+        route='order-book/predictions/v1/10',
         symbols=['BTC'],
-        start=datetime.now(tz=timezone.utc) - timedelta(hours=6),
-        end=datetime.now(tz=timezone.utc)
+        timeframe='1h',
+        since=datetime.now(tz=timezone.utc) - timedelta(hours=6),
+        till=datetime.now(tz=timezone.utc)
     ))
 
     # client.stream(
     #     routes={
-    #         'predictions/ml-order-book-model-1': ['BTC', 'LTC'],
+    #         'order-book/predictions/v1/10': ['BTC', 'LTC'],
     #     },
     # )
